#!/usr/bin/env python

"""
Synopsis:
    Various helper functions for lxml.objectify objects.
    Also, a command line harness for running several of those functions.
Usage:
    python objectify_helpers.py <command> <arg> <arg> ...
Commands:
    parse -- Parse XML doc and return doctree and root element.
        Arguments: XML doc file name.
    parse_with_schema -- Parse XML doc and return doctree and root element.
        Do validation against schema while parsing.
        Arguments: XML schema file name, XML doc file name
Examples:
    ./objectify_helpers.py parse <xmlinstancedoc>
    ./objectify_helpers.py --dump parse <xmlinstancedoc>
    ./objectify_helpers.py parse_with_schema <xmlschema> <xmlinstancedoc>
"""

import sys
import argparse
from lxml import etree
from lxml import objectify


Standard_attrs = set([
    '__dict__',
    '__getattr__',
    'addattr',
    'countchildren',
    'descendantpaths',
    '__class__',
    '__contains__',
    '__copy__',
    '__deepcopy__',
    '__delattr__',
    '__delitem__',
    '__doc__',
    '__format__',
    '__getattribute__',
    '__getitem__',
    '__hash__',
    '__init__',
    '__iter__',
    '__len__',
    '__new__',
    '__nonzero__',
    '__reduce__',
    '__reduce_ex__',
    '__repr__',
    '__reversed__',
    '__setattr__',
    '__setitem__',
    '__sizeof__',
    '__str__',
    '__subclasshook__',
    '_init',
    'addnext',
    'addprevious',
    'append',
    'attrib',
    'base',
    'clear',
    'extend',
    'find',
    'findall',
    'findtext',
    'get',
    'getchildren',
    'getiterator',
    'getnext',
    'getparent',
    'getprevious',
    'getroottree',
    'index',
    'insert',
    'items',
    'iter',
    'iterancestors',
    'iterchildren',
    'iterdescendants',
    'iterfind',
    'itersiblings',
    'itertext',
    'keys',
    'makeelement',
    'nsmap',
    'prefix',
    'remove',
    'replace',
    'set',
    'sourceline',
    'tag',
    'tail',
    'text',
    'values',
    'xpath',
])


def members(element):
    """Return a list of the names/tags of the members (children) of an element.
    """
    names = [attr for attr in dir(element) if attr not in Standard_attrs]
    return names


def sdump(element):
    """Produce a shallow dump of an objectify element.
    Like objectify.dump, except only one level deep.
    """
    content = objectify.dump(element)
    content = content.splitlines()
    prefix = '        '
    content = [line for line in content if not line.startswith(prefix)]
    content = '\n'.join(content)
    return content


def swrite(element, maxlevels=None, outfile=sys.stdout):
    """Recursively write out a formatted, readable representation of element.
    Possibly do shallow recursion.
    Limit recursion to maxlevels (default is all levels).
    Write output to file outfile (default is sys.stdout).
    """
    wrt = outfile.write
    _swrite(element, 0, maxlevels, wrt)


def _swrite(element, indent, maxlevels, wrt):
    indentstr = '    ' * indent
    wrt('{}{}: {}\n'.format(indentstr, element.tag, repr(element), ))
    for name, value in element.attrib.iteritems():
        wrt('  {}* {}: {}\n'.format(indentstr, name, value, ))
    indent += 1
    if maxlevels is not None and indent > maxlevels:
        return
    for child in element.iterchildren():
        _swrite(child, indent, maxlevels, wrt)


def children(element, tag=None):
    """Return a list of children of an element.
    Optional argument tag can be a single string or list of strings
    to select only children with that tag name.
    """
    child_list = [child.tag for child in element.iterchildren(tag=tag)]
    return child_list


def child_tags(element, tag=None):
    """Return a list of the tag names of the children of an element.
    Optional argument tag can be a single string or list of strings
    to select only children with that tag name.
    """
    tags = [child.tag for child in element.iterchildren(tag=tag)]
    return tags


def parse(infilename, options=None):
    """Parse an XML doc and return the objectify doctree and root element.
    If options.dump is True, print objectify.dump(root).
    """
    doc = objectify.parse(infilename)
    root = doc.getroot()
    if options is not None and options.dump:
        print objectify.dump(root)
    return doc, root


def parse_with_schema(schemaname, infilename, options=None):
    """Parse an XML doc with validateion against XML schema and
    return the objectify doctree and root element.
    If options.dump is True, print objectify.dump(root).
    """
    schema = etree.XMLSchema(file=schemaname)
    parser = objectify.makeparser(schema=schema)
    doc = objectify.parse(infilename, parser=parser)
    root = doc.getroot()
    if options is not None and options.dump:
        print objectify.dump(root)
    return doc, root


def main():
    parser = argparse.ArgumentParser(
        description=__doc__,
        formatter_class=argparse.RawDescriptionHelpFormatter,)
    parser.add_argument(
        'command', type=str,
        choices=['parse', 'parse_with_schema', ],
        help='command: one of parse, parse_with_schema')
    parser.add_argument(
        'args', type=str, nargs='*',
        help='arguments for the specified command.  See help.')
    parser.add_argument(
        '-d', '--dump', action='store_true', default=False,
        help='dump the objectify element and descendents.  default: False')
    options = parser.parse_args()
    if options.command == 'parse':
        if len(options.args) != 1:
            parser.error('need 1 argument (XML file name)')
        parse(options.args[0], options)
    elif options.command == 'parse_with_schema':
        if len(options.args) != 2:
            parser.error('need 2 arguments (schema and xml file)')
        parse_with_schema(options.args[0], options.args[1], options)


if __name__ == '__main__':
    #import ipdb; ipdb.set_trace()
    main()
